
%% Connect to Arduino
comPortList = get(handles.popupmenu1, 'String');
comPortSelected = get(handles.popupmenu1,'Value');
comPort = comPortList{comPortSelected};
sprintf('%d', comPort);
serialObject = SetupSerial(comPort, 115200);
set(handles.pushbutton2, 'UserData', serialObject);


%% Prepare for datalogging

% open a filestream
tNow = clock; % gets a 6-element vector of the current date and time in decimal form [year month day hour minute seconds]
tNow(6) = round(tNow(6)); % round to the nearest second
stringDateAndTime = ''; % initialize
for i = 1 : 1 : 6 % build up the time into a string
    stringDateAndTime = [stringDateAndTime,num2str(tNow(i),'%02d')];
    if i == 3 || i == 5 % after the day, and after the minute, add an underscore in the file name
        stringDateAndTime = [stringDateAndTime,'_']; % add an underscore (_)
    end
end
% check, whether data directory exists and create it in case it is missing
if exist('data', 'dir')~=7
    mkdir('data');
end
stringFileName = ['data/data_',stringDateAndTime,'.csv'];
idFile = fopen(stringFileName,'w');

%% Receive and plot data packets

hold on;


%% Settings for plotting
nIterationsToPlot = 50; % number of iterations to plot
hPlots = NaN(nIterationsToPlot, 3); % create an array of NaNs
firstRun = 1;
iSample = 1;
nRefreshRate = 1; % specify number of samples to receive before updating the plots

tData = zeros(nIterationsToPlot, 1);
rpmFanBuffer = zeros(nIterationsToPlot, 1);
tAmbientBuffer = zeros(nIterationsToPlot, 1);
tWindTunnelBuffer = zeros(nIterationsToPlot, 1);

nBadPacket = 0;
continueRun = true;


%% Receive and plot data from Arduino
startTime = tic;
while continueRun % continue, until stopButton has been pressed
    
    % check, whether the stop button has been pressed
    stopButtonData = get(handles.stopButton, 'UserData');
    continueRun = ~stopButtonData.stop; %when stopButton is pressed, its stopButtonData.stop value is changed from false to true
    
    % wait until a complete packet is in the buffer
    bytesAvailable = 0;
    while bytesAvailable < 8
        bytesAvailable = serialObject.BytesAvailable;
    end
    
    tData(nIterationsToPlot, 1) = toc(startTime);
    
    % process the incoming data
    [dataIntegrity, packet] = ReceivePacket(serialObject);
    
    % check on data integrity
    if dataIntegrity == false
        % increment the bad packet counter
        nBadPacket = nBadPacket + 1;
    else
        % Store incoming data
        for iArray = 2 : 1 : nIterationsToPlot
            tData(iArray - 1) = tData(iArray);
            rpmFanBuffer(iArray - 1) = rpmFanBuffer(iArray);
            tAmbientBuffer(iArray - 1) = tAmbientBuffer(iArray);
            tWindTunnelBuffer(iArray - 1) = tWindTunnelBuffer(iArray);
        end
        
        rpmFanBuffer(nIterationsToPlot, 1) = packet.rpmFan;
        tAmbientBuffer(nIterationsToPlot, 1) = packet.tAmbient;
        tWindTunnelBuffer(nIterationsToPlot, 1) = packet.tWindTunnel;        
        
        if iSample == nRefreshRate
            % redraw the plots
            handle1 = plot(handles.axes1, tData(:, 1), rpmFanBuffer(:, 1));
            handle2 = plot(handles.axes2, tData(:, 1), tAmbientBuffer(:, 1));
            handle3 = plot(handles.axes3, tData(:, 1), tWindTunnelBuffer(:, 1));
            
            % format the plots
            ylim(handles.axes1,[0, 6500])
            ylim(handles.axes2,[20, 50])
            ylim(handles.axes3,[20, 100])
            
            xlim(handles.axes1, [tData(1, 1), tData(nIterationsToPlot, 1)]);
            xlim(handles.axes2, [tData(1, 1), tData(nIterationsToPlot, 1)]);
            xlim(handles.axes3, [tData(1, 1), tData(nIterationsToPlot, 1)]);
            
            set(handle1, 'Color', [0 0 1], 'LineWidth', 3);
            set(handle2, 'Color', [0 0 1], 'LineWidth', 3);
            set(handle3, 'Color', [0 0 1], 'LineWidth', 3);
                     
            % plot legends and labels
%             legend(handles.axes1, 'RPM fan');
%             legend(handles.axes2, 'T ambient');
%             legend(handles.axes3, 'T wind tunnel');
%             
            title(handles.axes1,'RPM: fan', 'FontSize', 15, 'FontWeight', 'bold')
            xlabel(handles.axes1,'time [s]', 'FontSize', 15, 'FontWeight', 'bold')
            ylabel(handles.axes1,'rpm [1/min]', 'FontSize', 15, 'FontWeight', 'bold')
            
            title(handles.axes2,'Temperature: ambient', 'FontSize', 15, 'FontWeight', 'bold')
            xlabel(handles.axes2,'time [s]', 'FontSize', 15, 'FontWeight', 'bold')
            ylabel(handles.axes2,'T [C]', 'FontSize', 15, 'FontWeight', 'bold')
            
            title(handles.axes3,'Temperature: wind tunnel', 'FontSize', 15, 'FontWeight', 'bold')
            xlabel(handles.axes3,'time [s]', 'FontSize', 15, 'FontWeight', 'bold')
            ylabel(handles.axes3,'T [C]', 'FontSize', 15, 'FontWeight', 'bold')
        end
        
        % concatenate the individual data pieces into a single vector
        dataToLog = [tData(nIterationsToPlot, 1), bytesAvailable, nBadPacket, ...
            packet.rpmFan, packet.tAmbient, packet.tWindTunnel'];
        
        if firstRun == 1
            firstRun = 0;
            % write a header line to the file
            fprintf(idFile,['Fan Control data logger\n\n']);
            % print the column indices
            for i = 1 : 1 : length(dataToLog)
                fprintf(idFile, '%d,', i);
                if i == length(dataToLog)
                    fprintf(idFile, '\n');
                end
            end
            % print the column names
            fprintf(idFile,['Time stamp(s),' ...
                'nBytesInMATLABSerialBuffer, nBadPacket,'...
                'RPM fan [1/min], T ambient(C), T wind tunnel(C)\n']);
            
            % one-time format string creation
            stringFormat = ''; % initialize string
            for i = 1 : 1 : length(dataToLog);
                stringFormat = strcat(stringFormat, '%3.2f'); %append the proper # of format strings
                if i < length(dataToLog)
                    stringFormat = strcat(stringFormat, ','); %add in the commas for all but the last one
                else %i = length(data_to_log)
                    stringFormat = strcat(stringFormat, '\n'); %add in the new line command
                end
            end
        end
        
        % log the data into the file
        fprintf(idFile, stringFormat, dataToLog);
        
        % show the values in the GUI's textOutput textbox
        stringData = sprintf(['RPM fan = %d, T ambient = %0.2f,\n'...
            'T wind tunnel = %0.2f, t (s) = %0.3f,\n' ...
            'Bytes in MATLAB serial buffer = %2d,\n'...
            'Bad packet count = %1d'], ...
            packet.rpmFan, packet.tAmbient, packet.tWindTunnel, ...
            tData(nIterationsToPlot, 1), bytesAvailable, nBadPacket);
        set(handles.textOutput,'String', stringData);
        
        % increment sample counter
        iSample = iSample + 1;
        if iSample > nRefreshRate
            iSample = 1;
        end
    end
    pause(0.5);
end

fclose(serialObject);
delete(serialObject);
fclose(idFile); % close the filestream